@@ -12,7 +12,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 #ifndef _MYCOMPILER_
 #define _MYCOMPILER_
@@ -28,6 +28,16 @@
 #include <set>
 #include <libxml/xmlreader.h>
 
+#ifdef _MSC_VER
+  #if !defined(LTTOOLBOX_EXPORTS)
+    #define LTTOOLBOX_IMPORTS __declspec(dllimport)
+  #else
+    #define LTTOOLBOX_IMPORTS
+  #endif
+#else
+  #define LTTOOLBOX_IMPORTS
+#endif
+
 using namespace std;
 
 /**
@@ -97,6 +107,12 @@ private:
    */
   double default_weight;
 
+  /**
+   * Maintain morpheme boundaries
+   */
+  bool keep_boundaries;
+
+
   /**
    * Identifier of all the symbols during the compilation
    */
@@ -271,46 +287,49 @@ private:
    */
   bool allBlanks();
 
+  bool valid(wstring const& dir) const;
+
 public:
 
   /*
    * Constants to represent the element and the attributes of
    * dictionaries
    */
-  static wstring const COMPILER_DICTIONARY_ELEM;
-  static wstring const COMPILER_ALPHABET_ELEM;
-  static wstring const COMPILER_SDEFS_ELEM;
-  static wstring const COMPILER_SDEF_ELEM;
-  static wstring const COMPILER_N_ATTR;
-  static wstring const COMPILER_PARDEFS_ELEM;
-  static wstring const COMPILER_PARDEF_ELEM;
-  static wstring const COMPILER_PAR_ELEM;
-  static wstring const COMPILER_ENTRY_ELEM;
-  static wstring const COMPILER_RESTRICTION_ATTR;
-  static wstring const COMPILER_RESTRICTION_LR_VAL;
-  static wstring const COMPILER_RESTRICTION_RL_VAL;
-  static wstring const COMPILER_PAIR_ELEM;
-  static wstring const COMPILER_LEFT_ELEM;
-  static wstring const COMPILER_RIGHT_ELEM;
-  static wstring const COMPILER_S_ELEM;
-  static wstring const COMPILER_REGEXP_ELEM;
-  static wstring const COMPILER_SECTION_ELEM;
-  static wstring const COMPILER_ID_ATTR;
-  static wstring const COMPILER_TYPE_ATTR;
-  static wstring const COMPILER_IDENTITY_ELEM;
-  static wstring const COMPILER_IDENTITYGROUP_ELEM;
-  static wstring const COMPILER_JOIN_ELEM;
-  static wstring const COMPILER_BLANK_ELEM;
-  static wstring const COMPILER_POSTGENERATOR_ELEM;
-  static wstring const COMPILER_GROUP_ELEM;
-  static wstring const COMPILER_LEMMA_ATTR;
-  static wstring const COMPILER_IGNORE_ATTR;
-  static wstring const COMPILER_IGNORE_YES_VAL;
-  static wstring const COMPILER_ALT_ATTR;
-  static wstring const COMPILER_V_ATTR;
-  static wstring const COMPILER_VL_ATTR;
-  static wstring const COMPILER_VR_ATTR;
-  static wstring const COMPILER_WEIGHT_ATTR;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_DICTIONARY_ELEM;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_ALPHABET_ELEM;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_SDEFS_ELEM;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_SDEF_ELEM;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_N_ATTR;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_PARDEFS_ELEM;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_PARDEF_ELEM;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_PAR_ELEM;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_ENTRY_ELEM;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_RESTRICTION_ATTR;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_RESTRICTION_LR_VAL;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_RESTRICTION_RL_VAL;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_PAIR_ELEM;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_LEFT_ELEM;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_RIGHT_ELEM;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_S_ELEM;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_M_ELEM;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_REGEXP_ELEM;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_SECTION_ELEM;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_ID_ATTR;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_TYPE_ATTR;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_IDENTITY_ELEM;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_IDENTITYGROUP_ELEM;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_JOIN_ELEM;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_BLANK_ELEM;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_POSTGENERATOR_ELEM;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_GROUP_ELEM;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_LEMMA_ATTR;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_IGNORE_ATTR;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_IGNORE_YES_VAL;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_ALT_ATTR;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_V_ATTR;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_VL_ATTR;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_VR_ATTR;
+  LTTOOLBOX_IMPORTS static wstring const COMPILER_WEIGHT_ATTR;
 
   /**
    * Constructor
@@ -339,6 +358,11 @@ public:
    */
   void write(FILE *fd);
 
+  /**
+   * Set keep morpheme boundaries
+   */
+  void setKeepBoundaries(bool keep_boundaries = false);
+
   /**
    * Set verbose output
    */
