@@ -12,7 +12,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 #ifndef _EXPANDER_
 #define _EXPANDER_
@@ -39,38 +39,43 @@ private:
    * The libxml2's XML reader
    */
   xmlTextReaderPtr reader;
-  
+
   /**
    * The alt value
    */
   wstring alt;
-  
+
   /**
    * The variant value (monodix)
    */
   wstring variant;
-  
+
   /**
    * The variant value (left side of bidix)
    */
   wstring variant_left;
-  
+
   /**
    * The variant value (right side of bidix)
    */
   wstring variant_right;
-  
+
   /**
    * The paradigm being compiled
    */
   wstring current_paradigm;
-  
+
   /**
    * The direction of the compilation, 'lr' (left-to-right) or 'rl'
    * (right-to-left)
    */
   wstring direction;
-  
+
+  /**
+   * Do we print boundaries or not?
+   */
+  bool keep_boundaries;
+
   /**
    * Paradigms
    */
@@ -89,7 +94,7 @@ private:
    * Parse the &lt;pardef&gt; element
    */
   void procParDef();
-  
+
   /**
    * Parse the &lt;e&gt; element
    */
@@ -109,19 +114,19 @@ private:
   wstring attrib(wstring const &name);
 
   /**
-   * Parse the &lt;p&lt; element
+   * Parse the &lt;p&gt; element
    * @return a pair of strings, left part and right part of a transduction
    */
   pair<wstring, wstring> procTransduction();
 
   /**
-   * Parse the &lt;i&lt; element
+   * Parse the &lt;i&gt; element
    * @return a string from the dictionary's entry
    */
   wstring procIdentity();
 
   /**
-   * Parse the &lt;ig&lt; element
+   * Parse the &lt;ig&gt; element
    * @return a pair of strings, whose right part begins with '#'
    * but are otherwise identical
    */
@@ -145,13 +150,13 @@ private:
    * @param name the name of the node
    */
   void skipBlanks(wstring &name);
-  
-  
+
+
   void readString(wstring &result, wstring const &name);
-  
+
   /**
    * Force an element to be empty, and check for it
-   * @param name the element 
+   * @param name the element
    */
   void requireEmptyError(wstring const &name);
 
@@ -176,7 +181,7 @@ private:
    *               this method, the result of concatenations.
    * @param endings the endings to be appended.
    */
-  static void append(list<pair<wstring, wstring> > &result, 
+  static void append(list<pair<wstring, wstring> > &result,
                      list<pair<wstring, wstring> > const &endings);
 
   /**
@@ -185,7 +190,7 @@ private:
    *               this method, the result of concatenations.
    * @param endings the endings to be appended.
    */
-  static void append(list<pair<wstring, wstring> > &result, 
+  static void append(list<pair<wstring, wstring> > &result,
                      wstring const &endings);
 
   /**
@@ -194,8 +199,8 @@ private:
    *               this method, the result of concatenations.
    * @param endings the endings to be appended.
    */
-  static void append(list<pair<wstring, wstring> > &result, 
-	             pair<wstring, wstring> const &endings);
+  static void append(list<pair<wstring, wstring> > &result,
+                     pair<wstring, wstring> const &endings);
 
 public:
   /**
@@ -211,7 +216,8 @@ public:
   /**
    * Compile dictionary to letter transducers
    */
-  void expand(string const &fichero, FILE *output);
+  void expand(string const &file, FILE *output);
+
   /**
    * Set the alt value to use in compilation
    * @param a the value
@@ -235,7 +241,13 @@ public:
    * @param v the value
    */
    void setVariantRightValue(string const &v);
+
+  /**
+   * Set if we are going to keep morpheme boundaries
+   * @param keep true, false
+   */
+   void setKeepBoundaries(bool keep_boundaries = false);
+
 };
 
-
 #endif
