@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005 Universitat d'Alacant / Universidad de Alicante
+ * Copyright (C) 2005-2019 Universitat d'Alacant / Universidad de Alicante
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -12,7 +12,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef _FSTPROCESSOR_
@@ -103,6 +103,11 @@ private:
    */
   queue<wstring> blankqueue;
 
+  /**
+   * Queue of wordbound blanks, used in reading methods
+   */
+  queue<wstring> wblankqueue;
+
   /**
    * Set of characters being considered alphabetics
    */
@@ -227,6 +232,21 @@ private:
    */
   int maxAnalyses;
 
+  /**
+   * True if a wblank block ([[..]]xyz[[/]]) was just read
+   */
+  bool is_wblank;
+
+  /**
+   * True if skip_mode is false and need to collect wblanks
+   */
+  bool collect_wblanks;
+
+  /**
+   * True if a wblank has been processed for postgen and we need an ending wblank
+   */
+  bool need_end_wblank;
+
   /**
    * Output no more than 'N' best weight classes
    */
@@ -252,6 +272,20 @@ private:
    */
   wstring readFullBlock(FILE *input, wchar_t const delim1, wchar_t const delim2);
 
+  /**
+   * Reads a wordbound blank from the stream input
+   * @param input the stream being read
+   */
+  wstring readWblank(FILE *input);
+
+  /**
+   * Reads a wordbound blank (opening blank to closing blank) from the stream input -> [[...]]xyz[[/]]
+   * @param input the stream being read
+   * @param output the stream to write on
+   * @return true if the word enclosed by the wordbound blank has a ~ for postgeneration activation
+   */
+  bool wblankPostGen(FILE *input, FILE *output);
+
   /**
    * Returns true if the character code is identified as alphabetic
    * @param c the code provided by the user
@@ -267,15 +301,16 @@ private:
   bool isEscaped(wchar_t const c) const;
 
   /**
-   * Read text from stream (analysis version, also used in postgeneration)
+   * Read text from stream (analysis version)
    * @param input the stream to read
    * @return the next symbol in the stream
    */
   int readAnalysis(FILE *input);
 
   /**
-   * Read text from stream (generation version, also used in generation)
+   * Read text from stream (decomposition version)
    * @param input the stream to read
+   * @param output the stream to write on
    * @return the next symbol in the stream
    */
   int readDecomposition(FILE *input, FILE *output);
@@ -283,13 +318,15 @@ private:
   /**
    * Read text from stream (postgeneration version)
    * @param input the stream to read
+   * @param output the stream to write on
    * @return the next symbol in the stream
    */
-  int readPostgeneration(FILE *input);
+  int readPostgeneration(FILE *input, FILE *output);
 
   /**
    * Read text from stream (generation version)
    * @param input the stream to read
+   * @param output the stream being written to
    * @return the next symbol in the stream
    */
   int readGeneration(FILE *input, FILE *output);
@@ -297,6 +334,7 @@ private:
   /**
    * Read text from stream (biltrans version)
    * @param input the stream to read
+   * @param output the stream to write on
    * @return the queue of 0-symbols, and the next symbol in the stream
    */
   pair<wstring, int> readBilingual(FILE *input, FILE *output);
@@ -314,6 +352,26 @@ private:
    */
   void flushBlanks(FILE *output);
 
+  /**
+   * Flush all the wordbound blanks remaining in the current process
+   * @param output stream to write blanks
+   */
+  void flushWblanks(FILE *output);
+
+  /**
+   * Combine wordbound blanks in the queue and return them.
+   *
+   * May pop from 'wblankqueue' and set 'need_end_wblank' to true.
+   *
+   * If 'wblankqueue' (see which) is empty, we get an empty string,
+   * otherwise we return a semicolon-separated combination of opening
+   * wblanks in the queue. If there is only a closing wblank, we just
+   * set need_end_wblank.
+   *
+   * @return final wblank string
+  */
+  wstring combineWblanks();
+
   /**
    * Calculate the initial state of parsing
    */
@@ -331,6 +389,16 @@ private:
    */
   void writeEscaped(wstring const &str, FILE *output);
 
+  /**
+   * Write a string to an output stream.
+   * If we print a space, we may pop a space from blankqueue
+   * immediately, otherwise it should be printed afterwards.
+   *
+   * @param str the string to write, escaping characters
+   * @param output the stream to write in
+   * @return how many blanks to pop and print after printing lu
+   */
+  size_t writeEscapedPopBlanks(wstring const &str, FILE *output);
 
   /**
    * Write a string to an output stream, escaping all escapable characters
@@ -357,6 +425,16 @@ private:
    */
   void printWord(wstring const &sf, wstring const &lf, FILE *output);
 
+  /**
+   * Prints a word.
+   * If we print a space, we may pop a space from blankqueue.
+   *
+   * @param sf surface form of the word
+   * @param lf lexical form of the word
+   * @param output stream where the word is written
+   */
+  void printWordPopBlank(wstring const &sf, wstring const &lf, FILE *output);
+
   /**
    * Prints a word (Bilingual version)
    * @param sf surface form of the word
@@ -386,7 +464,15 @@ private:
   int readTMAnalysis(FILE *input);
 
   unsigned int lastBlank(wstring const &str);
+
+  /**
+   * Print one blankqueue item if there is one, or a given "space" value.
+   *
+   * @param val the space character to use if no blank queue
+   * @param output stream where the word is written
+   */
   void printSpace(wchar_t const val, FILE *output);
+
   void skipUntil(FILE *input, FILE *output, wint_t const character);
   static wstring removeTags(wstring const &str);
   wstring compoundAnalysis(wstring str, bool uppercase, bool firstupper);
@@ -394,7 +480,7 @@ private:
 
   void analysis_wrapper_null_flush(FILE *input, FILE *output);
   void lsx_wrapper_null_flush(FILE *input, FILE *output);
-  void bilingual_wrapper_null_flush(FILE *input, FILE *output);
+  void bilingual_wrapper_null_flush(FILE *input, FILE *output, GenerationMode mode = gm_unknown);
   void generation_wrapper_null_flush(FILE *input, FILE *output,
                                      GenerationMode mode);
   void postgeneration_wrapper_null_flush(FILE *input, FILE *output);
@@ -427,10 +513,9 @@ public:
   void postgeneration(FILE *input = stdin, FILE *output = stdout);
   void intergeneration(FILE *input = stdin, FILE *output = stdout);
   void transliteration(FILE *input = stdin, FILE *output = stdout);
-  void decomposition(FILE *input = stdin, FILE *output = stdout);
   wstring biltrans(wstring const &input_word, bool with_delim = true);
   wstring biltransfull(wstring const &input_word, bool with_delim = true);
-  void bilingual(FILE *input = stdin, FILE *output = stdout);
+  void bilingual(FILE *input = stdin, FILE *output = stdout, GenerationMode mode = gm_unknown);
   pair<wstring, int> biltransWithQueue(wstring const &input_word, bool with_delim = true);
   wstring biltransWithoutQueue(wstring const &input_word, bool with_delim = true);
   void SAO(FILE *input = stdin, FILE *output = stdout);
