@@ -12,7 +12,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 #ifndef _TRANSDUCTOR_
 #define _TRANSDUCTOR_
@@ -53,7 +53,7 @@ private:
 
   /**
    * Transitions of the transducer
-   * Schema: (souurce state, tag, target state, weight)
+   * Schema: (source state, tag, target state, weight)
    */
   map<int, multimap<int, pair<int, double> > > transitions;
 
@@ -81,6 +81,14 @@ private:
    * Empty transducer
    */
   void destroy();
+
+  /**
+   * Helper function for show()
+   * @param symbol the string to be escaped
+   * @param hfst if true, use HFST-compatible escape sequences
+   */
+  void escapeSymbol(wstring& symbol, bool hfst) const;
+
 public:
 
   /**
@@ -186,7 +194,15 @@ public:
    * @param epsilon_tag the tag to take as epsilon
    * @return the epsilon-connected states
    */
-  set<int> closure(int const state, int const epsilon_tag = 0);
+  set<int> closure(int const state, int const epsilon_tag = 0) const;
+
+  /**
+   * Returns the epsilon closure of a given state
+   * @param state the state
+   * @param epsilon_tags the tags to treat as epsilon
+   * @return the epsilon-connected states
+   */
+  set<int> closure(int const state, set<int> const &epsilon_tags) const;
 
   /**
    * Join all finals in one using epsilon transductions
@@ -213,8 +229,10 @@ public:
 
   /**
    * Print all the transductions of a transducer in ATT format
+   * @param hfst if true, use HFST-compatible escape characters
    * @param epsilon_tag the tag to take as epsilon
    */
+  void show(Alphabet const &a, FILE *output = stdout, int const epsilon_tag = 0, bool hfst = false) const;
   void show(Alphabet const &a, FILE *output = stdout, int const epsilon_tag = 0) const;
 
   /**
